using System;
using System.IO;
using UnityEditor;
using UnityEngine;
using UnityEditor.Animations;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Handles camera creation and animation setup.
    /// </summary>
    public static class CameraSetup
    {
        /// <summary>
        /// Sets up the camera in the scene based on the .omni file data.
        /// </summary>
        /// <param name="omniFilePath">The path to the .omni file.</param>
        /// <param name="omniData">Parsed OmniFile data.</param>
        /// <returns>The created camera GameObject.</returns>
        public static GameObject SetupCamera(string omniFilePath, OmniFile omniData)
        {
            // Extract camera animation path
            string cameraAnimationPath = omniData.data?.camera?.relative_path;
            if (!string.IsNullOrEmpty(cameraAnimationPath))
            {
                // Replace .abc extension with .fbx for camera animations
                if (cameraAnimationPath.EndsWith(".abc", StringComparison.OrdinalIgnoreCase))
                {
                    cameraAnimationPath = cameraAnimationPath.Replace(".abc", ".fbx", StringComparison.OrdinalIgnoreCase);
                    Debug.Log($"Replaced .abc extension with .fbx: {cameraAnimationPath}");
                }
            }
            else
            {
                Debug.LogError("Failed to extract camera animation path from .omni file.");
            }

            // Extract video resolution and determine orientation
            bool isPortrait = false;
            int videoWidth = 0;
            int videoHeight = 0;

            if (omniData.data?.video != null)
            {
                videoWidth = omniData.data.video.resolution.width;
                videoHeight = omniData.data.video.resolution.height;
                isPortrait = videoHeight > videoWidth;
                Debug.Log($"Video resolution: {videoWidth}x{videoHeight} - IsPortrait: {isPortrait}");
            }
            else
            {
                Debug.LogError("Failed to extract video resolution from .omni file.");
            }

            // Only set the Game View resolution if the user preference allows it
            if (OmniscientPreferences.ChangeResolutionOnImport)
            {
                SetGameViewResolution(videoWidth, videoHeight);
                Debug.Log("Game View resolution updated to match the imported video's resolution.");
            }
            else
            {
                Debug.Log("Skipping Game View resolution update due to user preferences.");
            }

            GameObject cameraObject = null;

            if (!string.IsNullOrEmpty(cameraAnimationPath))
            {
                string directoryPath = Path.GetDirectoryName(omniFilePath);
                string absoluteAnimationPath = Path.Combine(directoryPath, cameraAnimationPath);

                // Create a camera and apply the animation
                cameraObject = CreateCamera(isPortrait);
                ApplyCameraAnimation(absoluteAnimationPath, cameraObject);
            }
            else
            {
                Debug.LogError("Camera animation path is missing. Skipping camera animation setup.");
            }

            return cameraObject;
        }

        /// <summary>
        /// Creates a new camera in the Unity scene and adjusts its settings based on orientation.
        /// </summary>
        /// <param name="isPortrait">Flag indicating if the video is in portrait mode.</param>
        /// <returns>The created camera GameObject.</returns>
        private static GameObject CreateCamera(bool isPortrait)
        {
            // Disable existing MainCamera(s)
            Camera[] allCameras = Camera.allCameras;
            foreach (Camera cam in allCameras)
            {
                if (cam != null && cam.gameObject != null)
                {
                    cam.gameObject.SetActive(false);
                }
            }

            // Create a new camera object in the scene
            GameObject cameraObject = new GameObject("Camera_Omni");
            Camera cameraComponent = cameraObject.AddComponent<Camera>();
            cameraComponent.clearFlags = CameraClearFlags.SolidColor; // Set to Solid Color or Skybox as needed
            cameraComponent.backgroundColor = Color.black; // Set background color if not using a skybox

            // Enable Physical Camera
            cameraComponent.usePhysicalProperties = true;

            if (isPortrait)
            {
                // Swap sensor size to invert it
                Vector2 sensorSize = cameraComponent.sensorSize;
                cameraComponent.sensorSize = new Vector2(sensorSize.y, sensorSize.x);

                // Set gate fit to vertical
                cameraComponent.gateFit = Camera.GateFitMode.Vertical;

                Debug.Log("Adjusted camera for portrait orientation.");
            }
            else
            {
                // Ensure gate fit is set to horizontal for landscape
                cameraComponent.gateFit = Camera.GateFitMode.Horizontal;
                Debug.Log("Camera set for landscape orientation.");
            }

            // Set this camera as the MainCamera
            cameraObject.tag = "MainCamera";

            Debug.Log("Created camera object in the scene.");
            return cameraObject;
        }

        /// <summary>
        /// Applies animation to the specified camera.
        /// </summary>
        /// <param name="animationFilePath">The absolute path to the animation file.</param>
        /// <param name="cameraObject">The camera GameObject.</param>
        private static void ApplyCameraAnimation(string animationFilePath, GameObject cameraObject)
        {
            if (!File.Exists(animationFilePath))
            {
                Debug.LogError($"Animation file not found: {animationFilePath}");
                return;
            }

            string assetPath = PathHelper.EnsureFileIsInAssetsFolder(animationFilePath, "Animations");
            if (assetPath == null)
            {
                Debug.LogError("Failed to ensure animation file is in Assets.");
                return;
            }

            // Apply custom animation import settings
            ApplyAnimationImportSettings(assetPath);

            // Load the AnimationClip from the imported asset
            AnimationClip animationClip = AssetDatabase.LoadAssetAtPath<AnimationClip>(assetPath);

            if (animationClip == null)
            {
                Debug.LogError($"Failed to load animation clip from path: {assetPath}");
                return;
            }

            // Remove Interpolation
            RemoveInterpolationFromAnimationClip(animationClip);

            // Add Animator component to the camera and assign the animation
            Animator animator = cameraObject.GetComponent<Animator>();
            if (animator == null)
            {
                animator = cameraObject.AddComponent<Animator>();
            }

            // Create an Animator Controller and assign the animation clip
            string controllerPath = PathHelper.GenerateAssetPath("Omniscient/Animations", $"{Path.GetFileNameWithoutExtension(animationFilePath)}Controller.controller");

            // Check if the AnimatorController already exists
            RuntimeAnimatorController existingController = AssetDatabase.LoadAssetAtPath<RuntimeAnimatorController>(controllerPath);
            if (existingController == null)
            {
                // Create a new AnimatorController since it doesn't exist
                RuntimeAnimatorController animatorController = AnimatorControllerHelper.CreateSimpleAnimatorController(animationClip, controllerPath);
                animator.runtimeAnimatorController = animatorController;
                Debug.Log($"Created and applied AnimatorController to camera: {controllerPath}");
            }
            else
            {
                // AnimatorController already exists, use it
                animator.runtimeAnimatorController = existingController;
                Debug.Log($"AnimatorController already exists at path: {controllerPath}. Using existing controller.");
            }
        }

        /// <summary>
        /// Removes interpolation from all keyframes in the provided AnimationClip by setting their tangents to Step.
        /// </summary>
        /// <param name="animationClip">The AnimationClip to modify.</param>
        private static void RemoveInterpolationFromAnimationClip(AnimationClip animationClip)
        {
            // Get all curve bindings in the animation clip
            EditorCurveBinding[] bindings = AnimationUtility.GetCurveBindings(animationClip);

            foreach (var binding in bindings)
            {
                // Get the curve for the current binding
                AnimationCurve curve = AnimationUtility.GetEditorCurve(animationClip, binding);
                if (curve == null)
                    continue;

                // Retrieve all keyframes
                Keyframe[] keys = curve.keys;

                bool curveModified = false;

                // Iterate through each keyframe and set tangents to Step
                for (int i = 0; i < keys.Length; i++)
                {
                    Keyframe key = keys[i];

                    // Check if the current keyframe is already set to Step tangents
                    if (!IsKeyframeStep(key))
                    {
                        key.tangentMode = 8; // Step
                        key.inTangent = 0;
                        key.outTangent = 0;
                        keys[i] = key;
                        curveModified = true;
                    }
                }

                if (curveModified)
                {
                    // Assign the modified keys back to the curve
                    curve.keys = keys;
                    // Update the curve in the animation clip
                    AnimationUtility.SetEditorCurve(animationClip, binding, curve);
                    Debug.Log($"Set tangents to Step for curve: {binding.propertyName}");
                }
            }

            // Ensure the changes are saved
            EditorUtility.SetDirty(animationClip);
            AssetDatabase.SaveAssets();
        }

        /// <summary>
        /// Checks if a keyframe has Step tangents.
        /// </summary>
        /// <param name="key">The Keyframe to check.</param>
        /// <returns>True if both inTangent and outTangent are set to Step; otherwise, false.</returns>
        private static bool IsKeyframeStep(Keyframe key)
        {
            // In Unity, tangentMode uses bitmask where 8 represents Step
            // This is based on Unity's internal implementation and may change in future versions
            return key.tangentMode == 8;
        }

        /// <summary>
        /// Applies custom import settings to an animation asset.
        /// </summary>
        /// <param name="assetPath">The asset path of the animation.</param>
        private static void ApplyAnimationImportSettings(string assetPath)
        {
            // Get the ModelImporter for the asset
            ModelImporter importer = AssetImporter.GetAtPath(assetPath) as ModelImporter;
            if (importer == null)
            {
                Debug.LogError($"Failed to get ModelImporter for asset: {assetPath}");
                return;
            }

            // Disable Resample Curves
            importer.resampleCurves = false;

            // Disable Animation Compression
            importer.animationCompression = ModelImporterAnimationCompression.Off;

            // Apply the changes and reimport the asset
            importer.SaveAndReimport();

            Debug.Log($"Updated animation import settings for: {assetPath}");
        }

        /// <summary>
        /// Sets the Game View resolution to the specified width and height using GameViewUtils.
        /// </summary>
        /// <param name="width">Desired width.</param>
        /// <param name="height">Desired height.</param>
        private static void SetGameViewResolution(int width, int height)
        {
            // Define the desired size group type (Standalone is commonly used)
            GameViewSizeGroupType sizeGroupType = GameViewSizeGroupType.Standalone;

            // Define the desired size type (FixedResolution for exact width and height)
            GameViewUtils.GameViewSizeType sizeType = GameViewUtils.GameViewSizeType.FixedResolution;

            // Define the display text for the size
            string sizeText = $"{width}x{height}";

            // Check if the size already exists
            if (!GameViewUtils.SizeExists(sizeGroupType, sizeText))
            {
                // Add the custom size since it doesn't exist
                GameViewUtils.AddCustomSize(sizeType, sizeGroupType, width, height, sizeText);
                Debug.Log($"Added new Game View size: {sizeText}");
            }
            else
            {
                Debug.Log($"Game View size already exists: {sizeText}");
            }

            // Find the index of the desired size
            int sizeIndex = GameViewUtils.FindSize(sizeGroupType, sizeText);
            if (sizeIndex != -1)
            {
                // Set the Game View to the desired size
                GameViewUtils.SetSize(sizeIndex);
                Debug.Log($"Set Game View resolution to: {sizeText}");
            }
            else
            {
                Debug.LogError($"Failed to find or set Game View size: {sizeText}");
            }
        }
    }
}
