using System;
using System.IO;
using UnityEditor;
using UnityEngine;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Handles the import of geometry files into the Unity scene.
    /// </summary>
    public static class GeometryImporter
    {
        /// <summary>
        /// Imports geometry files as specified in the .omni file.
        /// </summary>
        /// <param name="omniFilePath">The path to the .omni file.</param>
        /// <param name="omniData">Parsed OmniFile data.</param>
        public static void ImportGeometry(string omniFilePath, OmniFile omniData)
        {
            if (omniData.data?.geometry?.relative_path != null && omniData.data.geometry.relative_path.Count > 0)
            {
                foreach (string geometryRelativePath in omniData.data.geometry.relative_path)
                {
                    string directoryPath = Path.GetDirectoryName(omniFilePath);
                    string absoluteGeometryPath = Path.Combine(directoryPath, geometryRelativePath);
                    Debug.Log($"Geometry file absolute path: {absoluteGeometryPath}");
                    ImportObjIntoSceneAsync(absoluteGeometryPath);
                }
            }
            else
            {
                Debug.LogError("Failed to extract geometry path from .omni file.");
            }
        }

        /// <summary>
        /// Imports an OBJ file into the Unity scene.
        /// </summary>
        /// <param name="objFilePath">The absolute path to the OBJ file.</param>
        private static void ImportObjIntoSceneAsync(string objFilePath)
        {
            if (!File.Exists(objFilePath))
            {
                Debug.LogError($"OBJ file not found: {objFilePath}");
                return;
            }

            string assetPath = PathHelper.EnsureFileIsInAssetsFolder(objFilePath, "Geometries");
            if (assetPath == null)
            {
                Debug.LogError("Failed to ensure OBJ file is in Assets.");
                return;
            }

            // Load the GameObject from the asset path
            GameObject existingObj = AssetDatabase.LoadAssetAtPath<GameObject>(assetPath);
            if (existingObj == null)
            {
                Debug.LogError($"Failed to load .obj file as GameObject from path: {assetPath}");
                return;
            }

            GameObject instantiatedObj = GameObject.Instantiate(existingObj);
            instantiatedObj.name = Path.GetFileNameWithoutExtension(objFilePath);

            if (instantiatedObj.GetComponent<MeshRenderer>() == null)
            {
                instantiatedObj.AddComponent<MeshRenderer>();
            }

            Debug.Log($"Successfully instantiated OBJ: {objFilePath}");
        }
    }
}
