using System;
using System.IO;
using System.Threading.Tasks;
using UnityEditor;
using UnityEditor.PackageManager;
using UnityEditor.PackageManager.Requests;
using UnityEngine;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Processes .omni files to import geometry, camera animations, and videos into the Unity scene.
    /// </summary>
    public class OmniFileProcessor
    {
        private static UnityEditor.PackageManager.PackageInfo currentPackageInfo;

        // Define the package name as per the package.json
        private const string PackageName = "com.stellaxis.omniscient";

        // Initialize the package info on load
        [InitializeOnLoadMethod]
        private static void OnLoad()
        {
            RefreshPackageInfoAsync();
        }

        /// <summary>
        /// Refreshes the current package information by initiating a list request.
        /// </summary>
        private static async void RefreshPackageInfoAsync()
        {
            var listRequest = Client.List();

            while (!listRequest.IsCompleted)
            {
                await Task.Delay(100);
            }

            if (listRequest.Status == StatusCode.Success)
            {
                foreach (var package in listRequest.Result)
                {
                    if (package.name.Equals(PackageName, StringComparison.OrdinalIgnoreCase))
                    {
                        currentPackageInfo = package;
                        Debug.Log($"Omniscient Plugin Version: {currentPackageInfo.version}");
                        break;
                    }
                }

                if (currentPackageInfo == null)
                {
                    Debug.LogError($"Package '{PackageName}' not found. Ensure that OmniFileProcessor is part of a package with a valid package.json.");
                }
            }
            else if (listRequest.Status >= StatusCode.Failure)
            {
                Debug.LogError($"Error retrieving package list: {listRequest.Error.message}");
            }
        }

        /// <summary>
        /// Initiates the import process for a given .omni file.
        /// </summary>
        /// <param name="omniFilePath">The absolute path to the .omni file.</param>
        public static void ImportOmniFileAsync(string omniFilePath)
        {
            try
            {
                EditorUtility.DisplayProgressBar("Importing .omni File", "Reading file...", 0.1f);

                // Step 1: Read and parse the .omni file
                string fileContent = File.ReadAllText(omniFilePath);
                OmniFile omniData = JsonUtility.FromJson<OmniFile>(fileContent);
                if (omniData == null)
                {
                    Debug.LogError("Failed to parse .omni file using JsonUtility.");
                    return;
                }

                // Step 2: Perform version checks
                if (!VersionChecker.PerformVersionChecks(omniData, currentPackageInfo))
                {
                    return; // Abort if versions are incompatible
                }

                EditorUtility.DisplayProgressBar("Importing .omni File", "Importing geometry...", 0.3f);

                // Step 3: Import geometry
                GeometryImporter.ImportGeometry(omniFilePath, omniData);

                EditorUtility.DisplayProgressBar("Importing .omni File", "Setting up camera...", 0.6f);

                // Step 4: Setup camera
                GameObject cameraObject = CameraSetup.SetupCamera(omniFilePath, omniData);

                EditorUtility.DisplayProgressBar("Importing .omni File", "Importing video...", 0.8f);

                // Step 5: Import video
                VideoImporter.ImportVideo(omniFilePath, omniData, cameraObject);
            }
            catch (Exception ex)
            {
                Debug.LogError($"An error occurred during import: {ex.Message}");
            }
            finally
            {
                EditorUtility.ClearProgressBar();
            }
        }
    }
}
