using UnityEditor;
using UnityEngine;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Provides a preferences panel in the Unity Editor where users can configure
    /// whether importing an .omni file updates the game display resolution or not.
    /// </summary>
    static class OmniscientPreferences
    {
        private const string PrefKey = "Omniscient_ChangeResolutionOnImport";
        private static readonly bool defaultChangeResolutionOnImportValue = true; // By default, resolution changes are enabled

        /// <summary>
        /// Gets or sets whether the plugin should change the game resolution on import.
        /// </summary>
        public static bool ChangeResolutionOnImport
        {
            get => EditorPrefs.GetBool(PrefKey, defaultChangeResolutionOnImportValue);
            set => EditorPrefs.SetBool(PrefKey, value);
        }

        [SettingsProvider]
        public static SettingsProvider CreateOmniscientSettingsProvider()
        {
            var provider = new SettingsProvider("Preferences/Omniscient", SettingsScope.User)
            {
                label = "Omniscient",
                guiHandler = (searchContext) =>
                {
                    EditorGUILayout.LabelField("Import Settings", EditorStyles.boldLabel);
                    EditorGUILayout.Space();

                    bool current = ChangeResolutionOnImport;
                    bool newVal = EditorGUILayout.Toggle(
                        new GUIContent("Change Resolution",
                            "If enabled, the Game View resolution will be updated to match the imported video's resolution."),
                        current);

                    if (newVal != current)
                    {
                        ChangeResolutionOnImport = newVal;
                        EditorPrefs.SetBool(PrefKey, newVal);
                    }
                }
            };

            return provider;
        }
    }
}
