using System;
using System.IO;
using System.Security.Cryptography;
using UnityEditor;
using UnityEngine;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Utility class for handling file system and Unity asset paths in a platform-agnostic manner.
    /// </summary>
    public static class PathHelper
    {
        /// <summary>
        /// Combines multiple strings into a single file system path.
        /// </summary>
        /// <param name="paths">Array of path segments.</param>
        /// <returns>The combined file system path.</returns>
        public static string Combine(params string[] paths)
        {
            return Path.Combine(paths);
        }

        /// <summary>
        /// Converts a file system path to a Unity asset path with forward slashes.
        /// </summary>
        /// <param name="path">The file system path.</param>
        /// <returns>The normalized Unity asset path.</returns>
        public static string ToUnityAssetPath(string path)
        {
            return path.Replace(Path.DirectorySeparatorChar, '/');
        }

        /// <summary>
        /// Generates a Unity asset path from subfolders and file name.
        /// </summary>
        /// <param name="subfolder">The subfolder within "Omniscient".</param>
        /// <param name="fileName">The file name.</param>
        /// <returns>The generated Unity asset path.</returns>
        public static string GenerateAssetPath(string subfolder, string fileName)
        {
            string combinedPath = Path.Combine("Assets", subfolder, fileName);
            return ToUnityAssetPath(combinedPath);
        }

        /// <summary>
        /// Ensures that a file is copied into the Assets folder under a specified subfolder within "Omniscient".
        /// </summary>
        /// <param name="filePath">The absolute path to the source file.</param>
        /// <param name="subfolder">The subfolder within "Omniscient" to copy the file to.</param>
        /// <returns>The asset path within Unity or null if copying failed.</returns>
        public static string EnsureFileIsInAssetsFolder(string filePath, string subfolder)
        {
            string fileName = Path.GetFileName(filePath);
            // Prepend "Omniscient/" to the subfolder
            string combinedSubfolder = Path.Combine("Omniscient", subfolder);
            string assetRelativePath = GenerateAssetPath(combinedSubfolder, fileName);
            string assetFullPath = Path.Combine(Application.dataPath, combinedSubfolder, fileName);

            // Ensure the destination folder exists
            string destinationFolder = Path.GetDirectoryName(assetFullPath);
            if (!Directory.Exists(destinationFolder))
            {
                Directory.CreateDirectory(destinationFolder);
                Debug.Log($"Created directory: {destinationFolder}");
            }

            // Check if the file already exists in the destination
            if (File.Exists(assetFullPath))
            {
                // Compare the source and destination files
                if (FilesAreIdentical(filePath, assetFullPath))
                {
                    Debug.Log($"File already exists and is identical at path: {assetRelativePath}. Skipping copy.");
                    return assetRelativePath;
                }
                else
                {
                    // Overwrite the existing file
                    try
                    {
                        File.Copy(filePath, assetFullPath, overwrite: true);
                        Debug.Log($"Overwrote existing file at path: {assetRelativePath}");
                        AssetDatabase.ImportAsset(assetRelativePath, ImportAssetOptions.ForceUpdate);
                    }
                    catch (Exception ex)
                    {
                        Debug.LogError($"Failed to overwrite file in Assets: {ex.Message}");
                        return null;
                    }
                }
            }
            else
            {
                // Copy the file since it doesn't exist
                try
                {
                    File.Copy(filePath, assetFullPath);
                    Debug.Log($"Copied file to Assets folder: {assetRelativePath}");
                    AssetDatabase.ImportAsset(assetRelativePath, ImportAssetOptions.ImportRecursive);
                }
                catch (Exception ex)
                {
                    Debug.LogError($"Failed to copy file to Assets: {ex.Message}");
                    return null;
                }
            }

            return assetRelativePath;
        }

        /// <summary>
        /// Compares two files to determine if they are identical.
        /// </summary>
        /// <param name="filePath1">The path to the first file.</param>
        /// <param name="filePath2">The path to the second file.</param>
        /// <returns>True if the files are identical, false otherwise.</returns>
        private static bool FilesAreIdentical(string filePath1, string filePath2)
        {
            using (var md5 = MD5.Create())
            {
                byte[] hash1;
                byte[] hash2;

                using (var stream1 = File.OpenRead(filePath1))
                {
                    hash1 = md5.ComputeHash(stream1);
                }

                using (var stream2 = File.OpenRead(filePath2))
                {
                    hash2 = md5.ComputeHash(stream2);
                }

                // Compare the two hash arrays
                return ByteArraysAreEqual(hash1, hash2);
            }
        }

        /// <summary>
        /// Compares two byte arrays for equality.
        /// </summary>
        /// <param name="a">First byte array.</param>
        /// <param name="b">Second byte array.</param>
        /// <returns>True if both arrays are identical; otherwise, false.</returns>
        private static bool ByteArraysAreEqual(byte[] a, byte[] b)
        {
            if (a.Length != b.Length)
                return false;

            for (int i = 0; i < a.Length; i++)
            {
                if (a[i] != b[i])
                    return false;
            }

            return true;
        }
    }
}
