// File: VersionChecker.cs

using System;
using UnityEditor;
using UnityEngine;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Handles version comparison and validation between the .omni file and the Unity plugin.
    /// </summary>
    public static class VersionChecker
    {
        /// <summary>
        /// Performs version checks between the .omni file and the Unity plugin.
        /// </summary>
        /// <param name="omniData">The parsed OmniFile data.</param>
        /// <param name="currentPackageInfo">Current package information.</param>
        /// <returns>True if versions are compatible; otherwise, false.</returns>
        public static bool PerformVersionChecks(OmniFile omniData, UnityEditor.PackageManager.PackageInfo currentPackageInfo)
        {
            if (currentPackageInfo == null)
            {
                Debug.LogError("Plugin version information is unavailable.");
                return false;
            }

            // Extract 'unity' section
            UnitySection unitySection = omniData.unity;
            if (unitySection == null)
            {
                // Inform the user that the .omni file is not compatible
                EditorUtility.DisplayDialog(
                    "Omniscient",
                    "The selected .omni file is not compatible with Unity. " +
                    "Please ensure that you select Unity when exporting the shot from the Omniscient app.",
                    "OK"
                );
                return false;
            }

            // Extract minimum and ideal plugin versions
            string minimumPluginVersion = unitySection.minimum_plugin_version;
            string idealPluginVersion = unitySection.ideal_plugin_version;

            if (string.IsNullOrEmpty(minimumPluginVersion) || string.IsNullOrEmpty(idealPluginVersion))
            {
                Debug.LogError("Failed to extract plugin version information from 'unity' section.");
                return false;
            }

            // Get the current plugin version dynamically
            string currentVersion = currentPackageInfo.version;
            if (string.IsNullOrEmpty(currentVersion))
            {
                Debug.LogError("Failed to retrieve current plugin version.");
                return false;
            }

            // Compare versions
            if (IsVersionLessThan(currentVersion, minimumPluginVersion))
            {
                // Current version is less than minimum required
                EditorUtility.DisplayDialog(
                    "Omniscient",
                    $"Current plugin version ({currentVersion}) is less than the minimum required version ({minimumPluginVersion}). Please update the plugin.",
                    "OK"
                );
                return false;
            }

            if (IsVersionLessThan(currentVersion, idealPluginVersion))
            {
                // Current version is less than ideal
                EditorUtility.DisplayDialog(
                    "Omniscient",
                    $"A newer plugin version ({idealPluginVersion}) is available. It's recommended to update for optimal performance.",
                    "OK"
                );
            }

            // Check the .omni file version
            string omniFileVersion = omniData.version;
            if (IsVersionLessThan(omniFileVersion, "2.2.0"))
            {
                EditorUtility.DisplayDialog(
                    "Omniscient",
                    "Please re-export the shot using the Omniscient app version 1.18 or later.",
                    "OK"
                );
                return false;
            }

            return true;
        }

        /// <summary>
        /// Compares two version strings.
        /// </summary>
        /// <param name="versionA">First version string.</param>
        /// <param name="versionB">Second version string.</param>
        /// <returns>True if versionA is less than versionB, false otherwise.</returns>
        private static bool IsVersionLessThan(string versionA, string versionB)
        {
            try
            {
                Version vA = new Version(versionA);
                Version vB = new Version(versionB);
                return vA.CompareTo(vB) < 0;
            }
            catch (Exception ex)
            {
                Debug.LogError($"Failed to parse version strings: '{versionA}' or '{versionB}'. Exception: {ex.Message}");
                return false;
            }
        }
    }
}
