using System;
using System.IO;
using UnityEditor;
using UnityEngine;
using UnityEngine.Video;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Handles the import of video files and sets them as backgrounds for the camera.
    /// </summary>
    public static class VideoImporter
    {
        /// <summary>
        /// Imports video files as specified in the .omni file and attaches them to the camera.
        /// </summary>
        /// <param name="omniFilePath">The path to the .omni file.</param>
        /// <param name="omniData">Parsed OmniFile data.</param>
        /// <param name="cameraObject">The camera GameObject.</param>
        public static void ImportVideo(string omniFilePath, OmniFile omniData, GameObject cameraObject)
        {
            if (omniData.data?.video != null && !string.IsNullOrEmpty(omniData.data.video.relative_path))
            {
                string directoryPath = Path.GetDirectoryName(omniFilePath);
                string absoluteVideoPath = Path.Combine(directoryPath, omniData.data.video.relative_path);
                Debug.Log($"Video file absolute path: {absoluteVideoPath}");
                ImportVideoIntoCameraBackgroundAsync(absoluteVideoPath, cameraObject, omniData.data.video.resolution.height > omniData.data.video.resolution.width);
            }
            else
            {
                Debug.LogError("Failed to extract video path from .omni file.");
            }
        }

        /// <summary>
        /// Imports a video file and sets it as the background for the specified camera.
        /// </summary>
        /// <param name="videoFilePath">The absolute path to the video file.</param>
        /// <param name="cameraObject">The camera GameObject.</param>
        /// <param name="isPortrait">Flag indicating if the video is in portrait mode.</param>
        private static void ImportVideoIntoCameraBackgroundAsync(string videoFilePath, GameObject cameraObject, bool isPortrait)
        {
            if (!File.Exists(videoFilePath))
            {
                Debug.LogError($"Video file not found: {videoFilePath}");
                return;
            }

            string assetPath = PathHelper.EnsureFileIsInAssetsFolder(videoFilePath, "Videos");
            if (assetPath == null)
            {
                Debug.LogError("Failed to ensure video file is in Assets.");
                return;
            }

            VideoClip videoClip = AssetDatabase.LoadAssetAtPath<VideoClip>(assetPath);

            if (videoClip == null)
            {
                Debug.LogError($"Failed to load VideoClip from path: {assetPath}");
                return;
            }

            if (cameraObject == null)
            {
                Debug.LogError("Camera object is null. Cannot attach VideoPlayer.");
                return;
            }

            // Add VideoPlayer component to the camera
            VideoPlayer videoPlayer = cameraObject.GetComponent<VideoPlayer>();
            if (videoPlayer == null)
            {
                videoPlayer = cameraObject.AddComponent<VideoPlayer>();
            }

            // Configure the VideoPlayer
            videoPlayer.clip = videoClip;
            videoPlayer.playOnAwake = true; // Play on Awake
            videoPlayer.waitForFirstFrame = false; // Do not wait for first frame
            videoPlayer.isLooping = false; // Do not loop
            videoPlayer.skipOnDrop = true; // Skip on drop
            videoPlayer.timeUpdateMode = VideoTimeUpdateMode.GameTime; // Use Game Time as update mode
            videoPlayer.renderMode = VideoRenderMode.CameraFarPlane; // Render as background

            if (isPortrait)
            {
                videoPlayer.aspectRatio = VideoAspectRatio.FitVertically; // Adjust for portrait
                Debug.Log("Set VideoPlayer for portrait orientation.");
            }
            else
            {
                videoPlayer.aspectRatio = VideoAspectRatio.FitHorizontally; // Adjust for landscape
                Debug.Log("Set VideoPlayer for landscape orientation.");
            }

            videoPlayer.targetCamera = cameraObject.GetComponent<Camera>();

            // Ensure the camera's clear flags are set appropriately
            Camera cam = cameraObject.GetComponent<Camera>();
            cam.clearFlags = CameraClearFlags.SolidColor;
            cam.backgroundColor = Color.black;

            // Start playing the video
            videoPlayer.Play();

            Debug.Log($"Successfully imported and set up video as camera background: {videoFilePath}");
        }
    }
}
